#include <string.h>
#include <glade/glade.h>
#include <gmpc/plugin.h>
#include <gdk/gdkkeysyms.h>
#include <config.h>
#include <gmpc/gmpc-mpddata-model.h>
#include <gmpc/gmpc-mpddata-treeview.h>
/* External pointer + function, there internal from gmpc */
extern GladeXML *pl3_xml;
int stop_song();
GtkWidget *extraplaylist = NULL;
static GtkWidget *extraplaylist_paned = NULL;
GtkWidget *ep_tree = NULL;
extern GtkTreeModel *playlist;

static void extra_playlist_song_changed(GtkTreeModel *model,GtkTreePath *path, GtkTreeIter *iter,gpointer data)
{
        if(cfg_get_single_value_as_int_with_default(config, "playlist", "st_cur_song", 0))
        {
            gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(ep_tree),
                    path,
                    NULL,
                    TRUE,0.5,0);
        }
}

static void extra_playlist_row_activated(GtkTreeView *tree,GtkTreePath *path, GtkTreeViewColumn *column, gpointer data)
{
	GtkTreeIter iter;
	GtkTreeModel *model = gtk_tree_view_get_model(tree);
	if(gtk_tree_model_get_iter(model, &iter,path))
	{
		gint pos = 0;
		gtk_tree_model_get(model, &iter, MPDDATA_MODEL_COL_SONG_ID, &pos, -1);
		mpd_player_play_id(connection,pos);
	}
}
static void extra_playlist_row_changed(GtkTreeSelection *sel, gpointer data)
{
    GtkTreeIter iter;
    GtkTreeModel *model = (GtkTreeModel *)playlist3_get_category_tree_store();
    if(extraplaylist == NULL)
        return;
    if(cfg_get_single_value_as_int_with_default(config,"extraplaylist", "enabled", 0) == 0)
        return;

    gtk_widget_hide(extraplaylist);
    if(gtk_tree_selection_get_selected(sel, &model, &iter))
    {
        gchar *name;
        /**
         * Hide when showing playlist
         */
        gtk_tree_model_get(model, &iter, PL3_CAT_ICON_ID, &name, -1);

        if(strcmp(name, "playlist-browser") == 0)
        {
            return;
        }
        gtk_widget_show_all(extraplaylist);

    }
    
}
static void extra_playlist_destroy() {
	if(extraplaylist)
	{
		int pos = gtk_paned_get_position(GTK_PANED(extraplaylist_paned));
		printf("pos is: %i\n",pos);
		if(pos>0)
			cfg_set_single_value_as_int(config, "extraplaylist", "paned-pos", pos);
	}
}
static void extraplaylist_delete_selected_songs()
{
	/* grab the selection from the tree */
	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(ep_tree));
	/* check if where connected */
	/* see if there is a row selected */
	if (gtk_tree_selection_count_selected_rows (selection) > 0)
	{
		GList *list = NULL, *llist = NULL;
		GtkTreeModel *model = GTK_TREE_MODEL(playlist);
		/* start a command list */
		/* grab the selected songs */
		list = gtk_tree_selection_get_selected_rows (selection, &model);
		/* grab the last song that is selected */
		llist = g_list_first (list);
		/* remove every selected song one by one */
		do{
			GtkTreeIter iter;
			int value;
			gtk_tree_model_get_iter (model, &iter,(GtkTreePath *) llist->data);
			gtk_tree_model_get (model, &iter, MPDDATA_MODEL_COL_SONG_ID, &value, -1);
			mpd_playlist_queue_delete_id(connection, value);			
		} while ((llist = g_list_next (llist)));

		/* close the list, so it will be executed */
		mpd_playlist_queue_commit(connection);
		/* free list */
		g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
		g_list_free (list);
	}
	else
	{
		/* create a warning message dialog */
		GtkWidget *dialog =
			gtk_message_dialog_new (GTK_WINDOW
					(glade_xml_get_widget
					 (pl3_xml, "pl3_win")),
					GTK_DIALOG_MODAL,
					GTK_MESSAGE_WARNING,
					GTK_BUTTONS_NONE,
					"Are you sure you want to clear the playlist?");
		gtk_dialog_add_buttons (GTK_DIALOG (dialog), GTK_STOCK_CANCEL,
				GTK_RESPONSE_CANCEL, GTK_STOCK_OK,
				GTK_RESPONSE_OK, NULL);
		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
				GTK_RESPONSE_CANCEL);

		switch (gtk_dialog_run (GTK_DIALOG (dialog)))
		{
			case GTK_RESPONSE_OK:
				/* check if where still connected */
				mpd_playlist_clear(connection);
		}
		gtk_widget_destroy (GTK_WIDGET (dialog));
	}
	/* update everything if where still connected */
	gtk_tree_selection_unselect_all(selection);

	mpd_status_queue_update(connection);
}
static int  extraplaylist_key_release_event(GtkWidget *widget, GdkEventKey *event)
{
	if(event->keyval == GDK_Delete)
	{
		extraplaylist_delete_selected_songs();
		return TRUE;
	}
	return FALSE;
}
static void extraplaylist_clear_playlist()
{
	mpd_playlist_clear(connection);
}
static void extraplaylist_crop_selected_songs()
{
	/* grab the selection from the tree */
	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(ep_tree));

	/* see if there is a row selected */	
	if (gtk_tree_selection_count_selected_rows (selection) > 0)
	{
		GtkTreeIter iter;


		/* start a command list */
		/* remove every selected song one by one */
		if(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playlist), &iter))
		{
			do{
				int value=0;
				if(!gtk_tree_selection_iter_is_selected(selection, &iter))
				{
					gtk_tree_model_get (GTK_TREE_MODEL(playlist), &iter, MPDDATA_MODEL_COL_SONG_ID, &value, -1);
					mpd_playlist_queue_delete_id(connection, value);				
				}
			} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(playlist),&iter));
			mpd_playlist_queue_commit(connection);
		}

	}
	/* update everything if where still connected */
	gtk_tree_selection_unselect_all(selection);

	mpd_status_queue_update(connection);
}
static void extraplaylist_browser_edit_columns(void)
{
  gmpc_mpddata_treeview_edit_columns(GMPC_MPDDATA_TREEVIEW(ep_tree));
}

static int extraplaylist_button_release_event(GtkTreeView *tree, GdkEventButton *event)
{
	if(event->button == 3)
	{
		/* del, crop */
		GtkWidget *item;
		GtkWidget *menu = gtk_menu_new();	
		/* add the delete widget */
		item = gtk_image_menu_item_new_from_stock(GTK_STOCK_REMOVE,NULL);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(extraplaylist_delete_selected_songs), NULL);


		/* add the delete widget */
		item = gtk_image_menu_item_new_with_label("Crop");
		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
				gtk_image_new_from_stock(GTK_STOCK_CUT, GTK_ICON_SIZE_MENU));
		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(extraplaylist_crop_selected_songs), NULL);		
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);

		gtk_menu_shell_append(GTK_MENU_SHELL(menu),gtk_separator_menu_item_new());
		/* add the clear widget */
		item = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLEAR,NULL);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(extraplaylist_clear_playlist), NULL);		
        item = gtk_image_menu_item_new_with_label("Edit Columns");
        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
                gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
        g_signal_connect(G_OBJECT(item), "activate",
        G_CALLBACK(extraplaylist_browser_edit_columns), NULL);


		gtk_widget_show_all(menu);
		gtk_menu_popup(GTK_MENU(menu), NULL, NULL,NULL, NULL, event->button, event->time);	
		return TRUE;
	}
	return FALSE;
}
static int extraplaylist_button_press_event(GtkTreeView *tree, GdkEventButton *event)
{
	GtkTreeSelection *sel = gtk_tree_view_get_selection(tree);
	if(event->button != 3 || gtk_tree_selection_count_selected_rows(sel) < 2|| !mpd_check_connected(connection))	
	{
		return FALSE;                                                                                           	
	}
	return TRUE;
}
static void extra_playlist_add() {
	GtkWidget *sw;

	if(pl3_xml == NULL) return;


	extraplaylist = gtk_event_box_new();
	gtk_container_set_border_width(GTK_CONTAINER(extraplaylist), 9);

    ep_tree = gmpc_mpddata_treeview_new("extra-playlist-tree", FALSE, GTK_TREE_MODEL(playlist));
	g_signal_connect(G_OBJECT(ep_tree), "row-activated", G_CALLBACK(extra_playlist_row_activated), NULL);
	sw = gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(sw), ep_tree);

    gtk_tree_view_set_reorderable(GTK_TREE_VIEW(ep_tree), TRUE);

	gtk_container_add(GTK_CONTAINER(extraplaylist), sw);

	/**
	 * Hack it into the main view
	 */

	GtkWidget *temp	= glade_xml_get_widget(pl3_xml, "hpaned1");
	g_object_ref(temp);
	extraplaylist_paned = gtk_vpaned_new();
	gtk_container_remove(GTK_CONTAINER(glade_xml_get_widget(pl3_xml,"vbox_control")),temp);



	gtk_paned_pack1(GTK_PANED(extraplaylist_paned), temp, TRUE, TRUE); 
	gtk_box_pack_start(GTK_BOX(glade_xml_get_widget(pl3_xml, "vbox_control")), extraplaylist_paned, TRUE, TRUE, 0);
	gtk_paned_pack2(GTK_PANED(extraplaylist_paned), extraplaylist, TRUE, TRUE); 

	gtk_paned_set_position(GTK_PANED(extraplaylist_paned),cfg_get_single_value_as_int_with_default(config, "extraplaylist", "paned-pos", 400));

	gtk_widget_show_all(extraplaylist_paned);
gtk_widget_hide(extraplaylist);




	GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(playlist3_get_category_tree_view()));
	g_signal_connect(G_OBJECT(sel), "changed",G_CALLBACK(extra_playlist_row_changed), NULL); 

	g_signal_connect(G_OBJECT(ep_tree), "key-press-event", G_CALLBACK(extraplaylist_key_release_event), NULL);
	g_signal_connect(G_OBJECT(ep_tree), "button-press-event", G_CALLBACK(extraplaylist_button_press_event), NULL);
	g_signal_connect(G_OBJECT(ep_tree), "button-release-event", G_CALLBACK(extraplaylist_button_release_event), NULL);
    g_signal_connect(G_OBJECT(playlist), "current_song_changed", G_CALLBACK(extra_playlist_song_changed), NULL);
	extra_playlist_row_changed(sel,NULL);
}


static void extra_playlist_init(void ) {
	if( cfg_get_single_value_as_int_with_default(config,"extraplaylist", "enabled", 1)) {
		gtk_init_add((GtkFunction )extra_playlist_add, NULL);
	}
}
static int get_enabled() {
	return cfg_get_single_value_as_int_with_default(config,"extraplaylist", "enabled", 0); 
}
static void set_enabled(int enable) {
	if(enable)
	{
		if(!extraplaylist) {
			extra_playlist_add();
		} else {
			gtk_widget_show_all(extraplaylist);
		}
		cfg_set_single_value_as_int(config,"extraplaylist", "enabled", enable);
	} else if(extraplaylist){
		cfg_set_single_value_as_int(config,"extraplaylist", "enabled", 0);
		gtk_widget_hide(extraplaylist);
	}
}

gmpcPlugin plugin = {
	.name = "Exta Playlist View",
	.version = {PLUGIN_MAJOR_VERSION, PLUGIN_MINOR_VERSION, PLUGIN_MICRO_VERSION},
	.plugin_type = GMPC_PLUGIN_NO_GUI,
	.init = extra_playlist_init,            /* initialization */
	.destroy = extra_playlist_destroy,         /* Destroy */
	.get_enabled = get_enabled,
	.set_enabled = set_enabled
};
int plugin_api_version = PLUGIN_API_VERSION;
